@@ -8,7 +8,7 @@ from .TwoSampleHC import binom_test_two_sided
 #from .TwoSampleHC import poisson_test_random
 
 __all__ = [
-	'HC'
+	'HC',
     'hc_vals',
     'two_sample_test',
     'two_sample_pvals',
